@@ -50,8 +50,8 @@ typedef struct Cop_T {
         int (*beginTransaction)(T C);
         int (*commit)(T C);
 	int (*rollback)(T C);
-	long long int (*lastRowId)(T C);
-	long long int (*rowsChanged)(T C);
+	long long (*lastRowId)(T C);
+	long long (*rowsChanged)(T C);
 	int (*execute)(T C, const char *sql, va_list ap);
 	ResultSet_T (*executeQuery)(T C, const char *sql, va_list ap);
         PreparedStatement_T (*prepareStatement)(T C, const char *sql, va_list ap);
