@@ -44,6 +44,7 @@
 #define T PreparedStatement_T
 struct PreparedStatement_S {
         Pop_T op;
+        int parameterCount;
         ResultSet_T resultSet;
         PreparedStatementDelegate_T D;
 };
@@ -65,13 +66,14 @@ static void clearResultSet(T P) {
 #pragma GCC visibility push(hidden)
 #endif
 
-T PreparedStatement_new(PreparedStatementDelegate_T D, Pop_T op) {
+T PreparedStatement_new(PreparedStatementDelegate_T D, Pop_T op, int parameterCount) {
 	T P;
 	assert(D);
 	assert(op);
-	NEW(P);
+        NEW(P);
 	P->D = D;
 	P->op = op;
+        P->parameterCount = parameterCount;
 	return P;
 }
 
@@ -88,7 +90,7 @@ void PreparedStatement_free(T *P) {
 #endif
 
 
-/* -------------------------------------------------------- Public methods */
+/* ------------------------------------------------------------ Parameters */
 
 
 void PreparedStatement_setString(T P, int parameterIndex, const char *x) {
@@ -103,7 +105,7 @@ void PreparedStatement_setInt(T P, int parameterIndex, int x) {
 }
 
 
-void PreparedStatement_setLLong(T P, int parameterIndex, long long int x) {
+void PreparedStatement_setLLong(T P, int parameterIndex, long long x) {
 	assert(P);
         P->op->setLLong(P->D, parameterIndex, x);
 }
@@ -121,6 +123,15 @@ void PreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size)
 }
 
 
+void PreparedStatement_setTimestamp(T P, int parameterIndex, time_t x) {
+        assert(P);
+        P->op->setTimestamp(P->D, parameterIndex, x);
+}
+
+
+/* -------------------------------------------------------- Public methods */
+
+
 void PreparedStatement_execute(T P) {
 	assert(P);
         clearResultSet(P);
@@ -138,8 +149,16 @@ ResultSet_T PreparedStatement_executeQuery(T P) {
 }
 
 
-long long int PreparedStatement_rowsChanged(T P) {
+long long PreparedStatement_rowsChanged(T P) {
         assert(P);
         return P->op->rowsChanged(P->D);
 }
 
+
+/* ------------------------------------------------------------ Properties */
+
+
+int PreparedStatement_getParameterCount(T P) {
+        assert(P);
+        return P->parameterCount;
+}
