@@ -25,6 +25,7 @@
 
 #ifndef PREPAREDSTATEMENT_INCLUDED
 #define PREPAREDSTATEMENT_INCLUDED
+#include <time.h>
 //<< Protected methods
 #include "PreparedStatementDelegate.h"
 //>> End Protected methods
@@ -93,6 +94,15 @@
  * the Prepared Statement is executed again or until the Connection is
  * returned to the Connection Pool. 
  *
+ * <h3>Date and Time</h3>
+ * PreparedStatement provides PreparedStatement_setTimestamp() for setting a
+ * Unix timestamp value. To set SQL Date, Time or DateTime values, simply use
+ * PreparedStatement_setString() with a time string format understood by your
+ * database. For instance to set a SQL Date value,
+ * <pre>
+ *   PreparedStatement_setString(p, "2013-12-28");
+ * </pre>
+ *
  * <i>A PreparedStatement is reentrant, but not thread-safe and should only be used by one thread (at the time).</i>
  * 
  * @see Connection.h ResultSet.h SQLException.h
@@ -109,9 +119,10 @@ typedef struct PreparedStatement_S *T;
  * Create a new PreparedStatement.
  * @param D the delegate used by this PreparedStatement
  * @param op delegate operations
+ * @param parameterCount number of parameters in this prepared statement
  * @return A new PreparedStatement object
  */
-T PreparedStatement_new(PreparedStatementDelegate_T D, Pop_T op);
+T PreparedStatement_new(PreparedStatementDelegate_T D, Pop_T op, int parameterCount);
 
 
 /**
@@ -122,6 +133,9 @@ void PreparedStatement_free(T *P);
 
 //>> End Protected methods
 
+/** @name Parameters */
+//@{
+
 /**
  * Sets the <i>in</i> parameter at index <code>parameterIndex</code> to the 
  * given string value. 
@@ -129,7 +143,7 @@ void PreparedStatement_free(T *P);
  * @param parameterIndex The first parameter is 1, the second is 2,..
  * @param x The string value to set. Must be a NUL terminated string. NULL
  * is allowed to indicate a SQL NULL value. 
- * @exception SQLException if a database access error occurs or if parameter 
+ * @exception SQLException If a database access error occurs or if parameter 
  * index is out of range
  * @see SQLException.h
 */
@@ -137,12 +151,18 @@ void PreparedStatement_setString(T P, int parameterIndex, const char *x);
 
 
 /**
- * Sets the <i>in</i> parameter at index <code>parameterIndex</code> to the 
+ * Sets the <i>in</i> parameter at index <code>parameterIndex</code> to the
  * given int value. 
+ * In general, on both 32 and 64 bits architecture, <code>int</code> is 4 bytes
+ * or 32 bits and <code>long long</code> is 8 bytes or 64 bits. A
+ * <code>long</code> type is usually equal to <code>int</code> on 32 bits
+ * architecture and equal to <code>long long</code> on 64 bits architecture.
+ * However, the width of integer types are architecture and compiler dependent.
+ * The above is usually true, but not necessarily.
  * @param P A PreparedStatement object
  * @param parameterIndex The first parameter is 1, the second is 2,..
  * @param x The int value to set
- * @exception SQLException if a database access error occurs or if parameter 
+ * @exception SQLException If a database access error occurs or if parameter
  * index is out of range
  * @see SQLException.h
  */
@@ -152,14 +172,20 @@ void PreparedStatement_setInt(T P, int parameterIndex, int x);
 /**
  * Sets the <i>in</i> parameter at index <code>parameterIndex</code> to the 
  * given long long value. 
+ * In general, on both 32 and 64 bits architecture, <code>int</code> is 4 bytes
+ * or 32 bits and <code>long long</code> is 8 bytes or 64 bits. A
+ * <code>long</code> type is usually equal to <code>int</code> on 32 bits
+ * architecture and equal to <code>long long</code> on 64 bits architecture.
+ * However, the width of integer types are architecture and compiler dependent.
+ * The above is usually true, but not necessarily.
  * @param P A PreparedStatement object
  * @param parameterIndex The first parameter is 1, the second is 2,..
  * @param x The long long value to set
- * @exception SQLException if a database access error occurs or if parameter 
+ * @exception SQLException If a database access error occurs or if parameter 
  * index is out of range
  * @see SQLException.h
  */
-void PreparedStatement_setLLong(T P, int parameterIndex, long long int x);
+void PreparedStatement_setLLong(T P, int parameterIndex, long long x);
 
 
 /**
@@ -168,7 +194,7 @@ void PreparedStatement_setLLong(T P, int parameterIndex, long long int x);
  * @param P A PreparedStatement object
  * @param parameterIndex The first parameter is 1, the second is 2,..
  * @param x The double value to set
- * @exception SQLException if a database access error occurs or if parameter 
+ * @exception SQLException If a database access error occurs or if parameter 
  * index is out of range
  * @see SQLException.h
  */
@@ -182,19 +208,38 @@ void PreparedStatement_setDouble(T P, int parameterIndex, double x);
  * @param parameterIndex The first parameter is 1, the second is 2,..
  * @param x The blob value to set
  * @param size The number of bytes in the blob 
- * @exception SQLException if a database access error occurs or if parameter 
+ * @exception SQLException If a database access error occurs or if parameter 
  * index is out of range
  * @see SQLException.h
  */
 void PreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size);
 
 
+/**
+ * Sets the <i>in</i> parameter at index <code>parameterIndex</code> to the
+ * given Unix timestamp value. The timestamp value given in <code>x</code>
+ * is expected to be in the UTC timezone. For instance, a value returned by
+ * time(3) which represents the system's notion of the current Greenwich time.
+ * <i class="textinfo">SQLite</i> does not have temporal SQL data types per se.
+ * Using this method with SQLite will store the timestamp value as a numerical
+ * type as-is.
+ * @param P A PreparedStatement object
+ * @param parameterIndex The first parameter is 1, the second is 2,..
+ * @param x The GMT timestamp value to set. E.g. a value returned by time(3)
+ * @exception SQLException If a database access error occurs or if parameter
+ * index is out of range
+ * @see SQLException.h ResultSet_getTimestamp
+ */
+void PreparedStatement_setTimestamp(T P, int parameterIndex, time_t x);
+
+//@}
+
 /**
  * Executes the prepared SQL statement, which may be an INSERT, UPDATE,
  * or DELETE statement or an SQL statement that returns nothing, such
  * as an SQL DDL statement. 
  * @param P A PreparedStatement object
- * @exception SQLException if a database error occurs
+ * @exception SQLException If a database error occurs
  * @see SQLException.h
  */
 void PreparedStatement_execute(T P);
@@ -208,7 +253,7 @@ void PreparedStatement_execute(T P);
  * @param P A PreparedStatement object
  * @return A ResultSet object that contains the data produced by the prepared
  * statement.
- * @exception SQLException if a database error occurs
+ * @exception SQLException If a database error occurs
  * @see ResultSet.h
  * @see SQLException.h
  */
@@ -217,14 +262,26 @@ ResultSet_T PreparedStatement_executeQuery(T P);
 
 /**
  * Returns the number of rows that was inserted, deleted or modified by the
- * most recently completed SQL statement on the database connection. If used 
+ * most recently completed SQL statement on the database connection. If used
  * with a transaction, this method should be called <i>before</i> commit is
  * executed, otherwise 0 is returned.
  * @param P A PreparedStatement object
  * @return The number of rows changed by the last (DIM) SQL statement
  */
-long long int PreparedStatement_rowsChanged(T P);
+long long PreparedStatement_rowsChanged(T P);
 
 
+/** @name Properties */
+//@{
+
+/**
+ * Returns the number of parameters in this prepared statement.
+ * @param P A PreparedStatement object
+ * @return The number of parameters in this prepared statement
+ */
+int PreparedStatement_getParameterCount(T P);
+
+//@}
+
 #undef T
 #endif
