@@ -3,12 +3,12 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
@@ -54,6 +54,7 @@ const struct Pop_T mysqlpops = {
         MysqlPreparedStatement_setInt,
         MysqlPreparedStatement_setLLong,
         MysqlPreparedStatement_setDouble,
+        MysqlPreparedStatement_setTimestamp,
         MysqlPreparedStatement_setBlob,
         MysqlPreparedStatement_execute,
         MysqlPreparedStatement_executeQuery,
@@ -62,9 +63,10 @@ const struct Pop_T mysqlpops = {
 
 typedef struct param_t {
         union {
-                long integer;
-                long long int llong;
+                int integer;
+                long long llong;
                 double real;
+                MYSQL_TIME timestamp;
         } type;
         long length;
 } *param_t;
@@ -73,18 +75,14 @@ typedef struct param_t {
 struct T {
         int maxRows;
         int lastError;
-        int paramCount;
         param_t params;
         MYSQL_STMT *stmt;
         MYSQL_BIND *bind;
+        int parameterCount;
 };
 
 static my_bool yes = true;
 
-#define TEST_INDEX \
-        int i; assert(P); i = parameterIndex - 1; if (P->paramCount <= 0 || \
-        i < 0 || i >= P->paramCount) THROW(SQLException, "Parameter index is out of range"); 
-
 extern const struct Rop_T mysqlrops;
 
 
@@ -95,16 +93,16 @@ extern const struct Rop_T mysqlrops;
 #pragma GCC visibility push(hidden)
 #endif
 
-T MysqlPreparedStatement_new(void *stmt, int maxRows) {
+T MysqlPreparedStatement_new(void *stmt, int maxRows, int parameterCount) {
         T P;
         assert(stmt);
         NEW(P);
         P->stmt = stmt;
         P->maxRows = maxRows;
-        P->paramCount = (int)mysql_stmt_param_count(P->stmt);
-        if (P->paramCount > 0) {
-                P->params = CALLOC(P->paramCount, sizeof(struct param_t));
-                P->bind = CALLOC(P->paramCount, sizeof(MYSQL_BIND));
+        P->parameterCount = parameterCount;
+        if (P->parameterCount > 0) {
+                P->params = CALLOC(P->parameterCount, sizeof(struct param_t));
+                P->bind = CALLOC(P->parameterCount, sizeof(MYSQL_BIND));
         }
         P->lastError = MYSQL_OK;
         return P;
@@ -116,7 +114,7 @@ void MysqlPreparedStatement_free(T *P) {
         FREE((*P)->bind);
         mysql_stmt_free_result((*P)->stmt);
 #if MYSQL_VERSION_ID >= 50503
-        /* In case the statement returns multiple result sets or in a stored procedure case, 
+        /* In case the statement returns multiple result sets or in a stored procedure case,
          think it does, we need to run them down. mysql_stmt_reset does not seem to work here. */
         while (mysql_stmt_next_result((*P)->stmt) == 0);
 #endif
@@ -127,7 +125,8 @@ void MysqlPreparedStatement_free(T *P) {
 
 
 void MysqlPreparedStatement_setString(T P, int parameterIndex, const char *x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->bind[i].buffer_type = MYSQL_TYPE_STRING;
         P->bind[i].buffer = (char*)x;
         if (! x) {
@@ -142,34 +141,55 @@ void MysqlPreparedStatement_setString(T P, int parameterIndex, const char *x) {
 
 
 void MysqlPreparedStatement_setInt(T P, int parameterIndex, int x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->params[i].type.integer = x;
         P->bind[i].buffer_type = MYSQL_TYPE_LONG;
-        P->bind[i].buffer = (char*)&P->params[i].type.integer;
+        P->bind[i].buffer = &P->params[i].type.integer;
         P->bind[i].is_null = 0;
 }
 
 
-void MysqlPreparedStatement_setLLong(T P, int parameterIndex, long long int x) {
-        TEST_INDEX
+void MysqlPreparedStatement_setLLong(T P, int parameterIndex, long long x) {
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->params[i].type.llong = x;
         P->bind[i].buffer_type = MYSQL_TYPE_LONGLONG;
-        P->bind[i].buffer = (char*)&P->params[i].type.llong;
+        P->bind[i].buffer = &P->params[i].type.llong;
         P->bind[i].is_null = 0;
 }
 
 
 void MysqlPreparedStatement_setDouble(T P, int parameterIndex, double x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->params[i].type.real = x;
         P->bind[i].buffer_type = MYSQL_TYPE_DOUBLE;
-        P->bind[i].buffer = (char*)&P->params[i].type.real;
+        P->bind[i].buffer = &P->params[i].type.real;
+        P->bind[i].is_null = 0;
+}
+
+
+void MysqlPreparedStatement_setTimestamp(T P, int parameterIndex, time_t x) {
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
+        struct tm ts = {.tm_isdst = -1};
+        gmtime_r(&x, &ts);
+        P->params[i].type.timestamp.year = ts.tm_year + 1900;
+        P->params[i].type.timestamp.month = ts.tm_mon + 1;
+        P->params[i].type.timestamp.day = ts.tm_mday;
+        P->params[i].type.timestamp.hour = ts.tm_hour;
+        P->params[i].type.timestamp.minute = ts.tm_min;
+        P->params[i].type.timestamp.second = ts.tm_sec;
+        P->bind[i].buffer_type = MYSQL_TYPE_TIMESTAMP;
+        P->bind[i].buffer = &P->params[i].type.timestamp;
         P->bind[i].is_null = 0;
 }
 
 
 void MysqlPreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->bind[i].buffer_type = MYSQL_TYPE_BLOB;
         P->bind[i].buffer = (void*)x;
         if (! x) {
@@ -185,13 +205,14 @@ void MysqlPreparedStatement_setBlob(T P, int parameterIndex, const void *x, int
 
 void MysqlPreparedStatement_execute(T P) {
         assert(P);
-        if ((P->paramCount > 0) && (P->lastError = mysql_stmt_bind_param(P->stmt, P->bind)))
+        if (P->parameterCount > 0)
+                if ((P->lastError = mysql_stmt_bind_param(P->stmt, P->bind)))
                         THROW(SQLException, "%s", mysql_stmt_error(P->stmt));
 #if MYSQL_VERSION_ID >= 50002
         unsigned long cursor = CURSOR_TYPE_NO_CURSOR;
         mysql_stmt_attr_set(P->stmt, STMT_ATTR_CURSOR_TYPE, &cursor);
 #endif
-        if ((P->lastError = mysql_stmt_execute(P->stmt))) 
+        if ((P->lastError = mysql_stmt_execute(P->stmt)))
                 THROW(SQLException, "%s", mysql_stmt_error(P->stmt));
         if (P->lastError == MYSQL_OK) {
                 /* Discard prepared param data in client/server */
@@ -202,13 +223,14 @@ void MysqlPreparedStatement_execute(T P) {
 
 ResultSet_T MysqlPreparedStatement_executeQuery(T P) {
         assert(P);
-        if ((P->paramCount > 0) && (P->lastError = mysql_stmt_bind_param(P->stmt, P->bind)))
-                THROW(SQLException, "%s", mysql_stmt_error(P->stmt));
+        if (P->parameterCount > 0)
+                if ((P->lastError = mysql_stmt_bind_param(P->stmt, P->bind)))
+                        THROW(SQLException, "%s", mysql_stmt_error(P->stmt));
 #if MYSQL_VERSION_ID >= 50002
         unsigned long cursor = CURSOR_TYPE_READ_ONLY;
         mysql_stmt_attr_set(P->stmt, STMT_ATTR_CURSOR_TYPE, &cursor);
 #endif
-        if ((P->lastError = mysql_stmt_execute(P->stmt))) 
+        if ((P->lastError = mysql_stmt_execute(P->stmt)))
                 THROW(SQLException, "%s", mysql_stmt_error(P->stmt));
         if (P->lastError == MYSQL_OK)
                 return ResultSet_new(MysqlResultSet_new(P->stmt, P->maxRows, true), (Rop_T)&mysqlrops);
@@ -217,9 +239,9 @@ ResultSet_T MysqlPreparedStatement_executeQuery(T P) {
 }
 
 
-long long int MysqlPreparedStatement_rowsChanged(T P) {
+long long MysqlPreparedStatement_rowsChanged(T P) {
         assert(P);
-        return (long long int)mysql_stmt_affected_rows(P->stmt);
+        return (long long)mysql_stmt_affected_rows(P->stmt);
 }
 
 #ifdef PACKAGE_PROTECTED
