@@ -114,7 +114,7 @@ static int doConnect(T C, URL_T url, char**  error) {
                 ERROR("no database specified in URL");
         ++database;
         /* Create a thread-safe OCI environment with N' substitution turned on. */
-        if (OCIEnvCreate(&C->env, OCI_THREADED | OCI_NCHAR_LITERAL_REPLACE_ON, 0, 0, 0, 0, 0, 0))
+        if (OCIEnvCreate(&C->env, OCI_THREADED | OCI_OBJECT | OCI_NCHAR_LITERAL_REPLACE_ON, 0, 0, 0, 0, 0, 0))
                 ERROR("Create a OCI environment failed");
         /* allocate an error handle */
         if (OCI_SUCCESS != OCIHandleAlloc(C->env, (dvoid**)&C->err, OCI_HTYPE_ERROR, 0, 0))
@@ -244,7 +244,7 @@ int  OracleConnection_rollback(T C) {
 }
 
 
-long long int OracleConnection_lastRowId(T C) {
+long long OracleConnection_lastRowId(T C) {
         /*:FIXME:*/
         /*
          Oracle's RowID can be mapped on string only
@@ -274,7 +274,7 @@ long long int OracleConnection_lastRowId(T C) {
 }
 
 
-long long int OracleConnection_rowsChanged(T C) {
+long long OracleConnection_rowsChanged(T C) {
         assert(C);
         return C->rowsChanged;
 }
@@ -285,9 +285,8 @@ int  OracleConnection_execute(T C, const char *sql, va_list ap) {
         va_list ap_copy;
         assert(C);
         C->rowsChanged = 0;
-        StringBuffer_clear(C->sb);
         va_copy(ap_copy, ap);
-        StringBuffer_vappend(C->sb, sql, ap_copy);
+        StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
         StringBuffer_trim(C->sb);
         /* Build statement */
@@ -321,9 +320,8 @@ ResultSet_T OracleConnection_executeQuery(T C, const char *sql, va_list ap) {
         va_list  ap_copy;
         assert(C);
         C->rowsChanged = 0;
-        StringBuffer_clear(C->sb);
         va_copy(ap_copy, ap);
-        StringBuffer_vappend(C->sb, sql, ap_copy);
+        StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
         StringBuffer_trim(C->sb);
         /* Build statement */
@@ -347,7 +345,7 @@ ResultSet_T OracleConnection_executeQuery(T C, const char *sql, va_list ap) {
         C->lastError = OCIAttrGet(stmtp, OCI_HTYPE_STMT, &C->rowsChanged, 0, OCI_ATTR_ROW_COUNT, C->err);
         if (C->lastError != OCI_SUCCESS && C->lastError != OCI_SUCCESS_WITH_INFO)
                 DEBUG("OracleConnection_execute: Error in OCIAttrGet %d (%s)\n", C->lastError, OracleConnection_getLastError(C));
-        return ResultSet_new(OracleResultSet_new(stmtp, C->env, C->err, C->svc, true, C->maxRows), (Rop_T)&oraclerops);
+        return ResultSet_new(OracleResultSet_new(stmtp, C->env, C->usr, C->err, C->svc, true, C->maxRows), (Rop_T)&oraclerops);
 }
 
 
@@ -355,12 +353,11 @@ PreparedStatement_T OracleConnection_prepareStatement(T C, const char *sql, va_l
         OCIStmt *stmtp;
         va_list ap_copy;
         assert(C);
-        StringBuffer_clear(C->sb);
         va_copy(ap_copy, ap);
-        StringBuffer_vappend(C->sb, sql, ap_copy);
+        StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
         StringBuffer_trim(C->sb);
-        StringBuffer_prepare4oracle(C->sb);
+        int paramCount = StringBuffer_prepare4oracle(C->sb);
         /* Build statement */
         C->lastError = OCIHandleAlloc(C->env, (void **)&stmtp, OCI_HTYPE_STMT, 0, 0);
         if (C->lastError != OCI_SUCCESS && C->lastError != OCI_SUCCESS_WITH_INFO)
@@ -370,7 +367,7 @@ PreparedStatement_T OracleConnection_prepareStatement(T C, const char *sql, va_l
                 OCIHandleFree(stmtp, OCI_HTYPE_STMT);
                 return NULL;
         }
-        return PreparedStatement_new(OraclePreparedStatement_new(stmtp, C->env, C->err, C->svc, C->maxRows), (Pop_T)&oraclepops);
+        return PreparedStatement_new(OraclePreparedStatement_new(stmtp, C->env, C->usr, C->err, C->svc, C->maxRows), (Pop_T)&oraclepops, paramCount);
 }
 
 
@@ -410,7 +407,7 @@ const char *OracleConnection_getLastError(T C) {
 
 /* Oracle client library finalization */
 void OracleConnection_onstop(void) {
-        // Not needed, OracleConnection_free below handle finalization
+        // Not needed, OracleConnection_free handle finalization
 }
 
 #ifdef PACKAGE_PROTECTED
