@@ -33,8 +33,8 @@ int  OracleConnection_ping(T C);
 int  OracleConnection_beginTransaction(T C);
 int  OracleConnection_commit(T C);
 int  OracleConnection_rollback(T C);
-long long int OracleConnection_lastRowId(T C);
-long long int OracleConnection_rowsChanged(T C);
+long long OracleConnection_lastRowId(T C);
+long long OracleConnection_rowsChanged(T C);
 int  OracleConnection_execute(T C, const char *sql, va_list ap);
 ResultSet_T OracleConnection_executeQuery(T C, const char *sql, va_list ap);
 PreparedStatement_T OracleConnection_prepareStatement(T C, const char *sql, va_list ap);
