@@ -56,10 +56,13 @@ const struct Rop_T oraclerops = {
         OracleResultSet_free,
         OracleResultSet_getColumnCount,
         OracleResultSet_getColumnName,
-        OracleResultSet_next,
         OracleResultSet_getColumnSize,
+        OracleResultSet_next,
+        OracleResultSet_isnull,
         OracleResultSet_getString,
         OracleResultSet_getBlob,
+        NULL, // getTimestamp is handled in ResultSet
+        NULL  // getDateTime is handled in ResultSet
 };
 typedef struct column_t {
         OCIDefine *def;
@@ -68,6 +71,7 @@ typedef struct column_t {
         char *name;
         unsigned long length;
         OCILobLocator *lob_loc;
+        OCIDateTime   *date; 
 } *column_t;
 #define T ResultSetDelegate_T
 struct T {
@@ -76,6 +80,7 @@ struct T {
         ub4         maxRow;
         OCIStmt*    stmt;
         OCIEnv*     env;
+        OCISession* usr;
         OCIError*   err;
         OCISvcCtx*  svc;
         column_t    columns;
@@ -87,9 +92,7 @@ struct T {
 #define ORACLE_COLUMN_NAME_LOWERCASE 2
 #endif
 #define LOB_CHUNK_SIZE  2000
-#define TEST_INDEX \
-        int i; assert(R);i = columnIndex-1; if (R->columnCount <= 0 || \
-        i < 0 || i >= R->columnCount) { THROW(SQLException, "Column index is out of range"); } 
+#define DATE_STR_BUF_SIZE   255
 
 
 /* ------------------------------------------------------- Private methods */
@@ -139,6 +142,18 @@ static int initaleDefiningBuffers(T R) {
                                 R->lastError = OCIDefineByPos(R->stmt, &R->columns[i-1].def, R->err, i, 
                                         &(R->columns[i-1].lob_loc), deptlen, SQLT_CLOB, &(R->columns[i-1].isNull), 0, 0, OCI_DEFAULT);
                                 break;
+                        case SQLT_DAT:
+                        case SQLT_DATE:
+                        case SQLT_TIMESTAMP:
+                        case SQLT_TIMESTAMP_TZ:
+                        case SQLT_TIMESTAMP_LTZ:
+                                R->columns[i-1].buffer = NULL;
+                                status = OCIDescriptorAlloc((dvoid *)R->env, (dvoid **) &(R->columns[i-1].date),
+                                                (ub4) OCI_DTYPE_TIMESTAMP,
+                                                (size_t) 0, (dvoid **) 0);
+                                R->lastError = OCIDefineByPos(R->stmt, &R->columns[i-1].def, R->err, i, 
+                                        &(R->columns[i-1].date), sizeof(R->columns[i-1].date), SQLT_TIMESTAMP, &(R->columns[i-1].isNull), 0, 0, OCI_DEFAULT);
+                                break;
                         default:
                                 R->columns[i-1].lob_loc = NULL;
                                 R->columns[i-1].buffer = ALLOC(deptlen + 1);
@@ -148,15 +163,21 @@ static int initaleDefiningBuffers(T R) {
                 {
                         char *col_name;
                         ub4   col_name_len;
+                        char* tmp_buffer;
 
                         R->lastError = OCIAttrGet(pard, OCI_DTYPE_PARAM, &col_name, &col_name_len, OCI_ATTR_NAME, R->err);
                         if (R->lastError != OCI_SUCCESS)
                                 continue;
+                        // column name could be non NULL terminated
+                        // it is not allowed to do: col_name[col_name_len] = 0;
+                        // so, copy the string
+                        tmp_buffer = Str_ndup(col_name, col_name_len);
 #if defined(ORACLE_COLUMN_NAME_LOWERCASE) && ORACLE_COLUMN_NAME_LOWERCASE > 1
-                        R->columns[i-1].name = CALLOC(1, col_name_len);
-                        OCIMultiByteStrCaseConversion(R->env, R->columns[i-1].name, col_name, OCI_NLS_LOWERCASE);
+                        R->columns[i-1].name = CALLOC(1, col_name_len+1);
+                        OCIMultiByteStrCaseConversion(R->env, R->columns[i-1].name, tmp_buffer, OCI_NLS_LOWERCASE);
+                        FREE(tmp_buffer);
 #else
-                        R->columns[i-1].name = Str_dup(col_name);
+                        R->columns[i-1].name = tmp_buffer;
 #endif /*COLLUMN_NAME_LOWERCASE*/
                 }
                 OCIDescriptorFree(pard, OCI_DTYPE_PARAM);
@@ -167,6 +188,25 @@ static int initaleDefiningBuffers(T R) {
         return true;
 }
 
+static int OracleDate_toString(T R, int i)
+{
+        const char fmt[] = "IYYY-MM-DD HH24.MI.SS"; // "YYYY-MM-DD HH24:MI:SS TZR TZD"
+
+        R->columns[i].length = DATE_STR_BUF_SIZE;
+        if (R->columns[i].buffer)
+                FREE(R->columns[i].buffer);
+
+        R->columns[i].buffer = ALLOC(R->columns[i].length + 1);
+        R->lastError = OCIDateTimeToText(R->usr, 
+                                         R->err, 
+                                         R->columns[i].date,
+                                         fmt, strlen(fmt),
+                                         0,
+                                         NULL, 0,
+                                         (ub4*)&(R->columns[i].length), (OraText *)R->columns[i].buffer);
+        return ((R->lastError == OCI_SUCCESS) || (R->lastError == OCI_SUCCESS_WITH_INFO));;
+}
+
 
 /* ----------------------------------------------------- Protected methods */
 
@@ -175,7 +215,7 @@ static int initaleDefiningBuffers(T R) {
 #pragma GCC visibility push(hidden)
 #endif
 
-T OracleResultSet_new(OCIStmt *stmt, OCIEnv *env, OCIError *err, OCISvcCtx *svc, int need_free, int max_row) {
+T OracleResultSet_new(OCIStmt *stmt, OCIEnv *env, OCISession* usr, OCIError *err, OCISvcCtx *svc, int need_free, int max_row) {
         T R;
         assert(stmt);
         assert(env);
@@ -186,6 +226,7 @@ T OracleResultSet_new(OCIStmt *stmt, OCIEnv *env, OCIError *err, OCISvcCtx *svc,
         R->env  = env;
         R->err  = err;
         R->svc  = svc;
+        R->usr  = usr;
         R->freeStatement = need_free;
         R->row = 0;
         R->lastError = OCIAttrGet(R->stmt, OCI_HTYPE_STMT, &R->maxRow, NULL, OCI_ATTR_ROW_COUNT/*OCI_ATTR_ROWS_FETCHED*/, R->err);
@@ -213,6 +254,8 @@ void OracleResultSet_free(T *R) {
         for (int i = 0; i < (*R)->columnCount; i++) {
                 if ((*R)->columns[i].lob_loc)
                         OCIDescriptorFree((*R)->columns[i].lob_loc, OCI_DTYPE_LOB);
+                if ((*R)->columns[i].date)
+                        OCIDescriptorFree((dvoid*)(*R)->columns[i].date, OCI_DTYPE_TIMESTAMP);
                 FREE((*R)->columns[i].buffer);
                 FREE((*R)->columns[i].name);
         }
@@ -235,22 +278,6 @@ const char *OracleResultSet_getColumnName(T R, int column) {
 }
 
 
-int OracleResultSet_next(T R) {
-        assert(R);
-        if ((R->row < 0) || ((R->maxRow > 0) && (R->row >= R->maxRow)))
-                return false;
-        R->lastError = OCIStmtFetch2(R->stmt, R->err, 1, OCI_FETCH_NEXT, 0, OCI_DEFAULT);
-        if (R->lastError == OCI_NO_DATA) 
-                return false;
-        if (R->lastError != OCI_SUCCESS && R->lastError != OCI_SUCCESS_WITH_INFO)
-                THROW(SQLException, "%s", OraclePreparedStatement_getLastError(R->lastError, R->err));
-	if (R->lastError == OCI_SUCCESS_WITH_INFO)
-		DEBUG("OracleResultSet_next Error %d, '%s'\n", R->lastError, OraclePreparedStatement_getLastError(R->lastError, R->err));
-        R->row++;
-        return ((R->lastError == OCI_SUCCESS) || (R->lastError == OCI_SUCCESS_WITH_INFO));
-}
-
-
 long OracleResultSet_getColumnSize(T R, int columnIndex) {
         OCIParam* pard = NULL;
         ub4 char_semantics = 0;
@@ -274,10 +301,41 @@ long OracleResultSet_getColumnSize(T R, int columnIndex) {
 }
 
 
+int OracleResultSet_next(T R) {
+        assert(R);
+        if ((R->row < 0) || ((R->maxRow > 0) && (R->row >= R->maxRow)))
+                return false;
+        R->lastError = OCIStmtFetch2(R->stmt, R->err, 1, OCI_FETCH_NEXT, 0, OCI_DEFAULT);
+        if (R->lastError == OCI_NO_DATA) 
+                return false;
+        if (R->lastError != OCI_SUCCESS && R->lastError != OCI_SUCCESS_WITH_INFO)
+                THROW(SQLException, "%s", OraclePreparedStatement_getLastError(R->lastError, R->err));
+        if (R->lastError == OCI_SUCCESS_WITH_INFO)
+                DEBUG("OracleResultSet_next Error %d, '%s'\n", R->lastError, OraclePreparedStatement_getLastError(R->lastError, R->err));
+        R->row++;
+        return ((R->lastError == OCI_SUCCESS) || (R->lastError == OCI_SUCCESS_WITH_INFO));
+}
+
+
+int OracleResultSet_isnull(T R, int columnIndex) {
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        return R->columns[i].isNull;
+}
+
+
 const char *OracleResultSet_getString(T R, int columnIndex) {
-        TEST_INDEX
-        if (R->columns[i].isNull) 
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        if (R->columns[i].isNull)
                 return NULL;
+        if (R->columns[i].date)
+        {
+                if (!OracleDate_toString(R, i))
+                {
+                        THROW(SQLException, "%s", OraclePreparedStatement_getLastError(R->lastError, R->err));
+                }
+        }
         if (R->columns[i].buffer)
                 R->columns[i].buffer[R->columns[i].length] = 0;
         return R->columns[i].buffer;
@@ -285,8 +343,9 @@ const char *OracleResultSet_getString(T R, int columnIndex) {
 
 
 const void *OracleResultSet_getBlob(T R, int columnIndex, int *size) {
-        TEST_INDEX
-        if (R->columns[i].isNull) 
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        if (R->columns[i].isNull)
                 return NULL;
         if (R->columns[i].buffer)
                 FREE(R->columns[i].buffer);
@@ -304,7 +363,7 @@ const void *OracleResultSet_getBlob(T R, int columnIndex, int *size) {
                 if (read_bytes) {
                         total_bytes += read_bytes;
                         piece = OCI_NEXT_PIECE;
-                        R->columns[i].buffer = RESIZE(R->columns[i].buffer, total_bytes + LOB_CHUNK_SIZE);
+                        R->columns[i].buffer = RESIZE(R->columns[i].buffer, (long)(total_bytes + LOB_CHUNK_SIZE));
                 }
         } while (R->lastError == OCI_NEED_DATA);
         if (R->lastError != OCI_SUCCESS && R->lastError != OCI_SUCCESS_WITH_INFO) {
