@@ -29,6 +29,7 @@
 #include <string.h>
 #include <libpq-fe.h>
 
+#include "system/Time.h"
 #include "ResultSet.h"
 #include "PostgresqlResultSet.h"
 #include "PreparedStatementDelegate.h"
@@ -54,6 +55,7 @@ const struct Pop_T postgresqlpops = {
         PostgresqlPreparedStatement_setInt,
         PostgresqlPreparedStatement_setLLong,
         PostgresqlPreparedStatement_setDouble,
+        PostgresqlPreparedStatement_setTimestamp,
         PostgresqlPreparedStatement_setBlob,
         PostgresqlPreparedStatement_execute,
         PostgresqlPreparedStatement_executeQuery,
@@ -77,10 +79,6 @@ struct T {
         param_t params;
 };
 
-#define TEST_INDEX \
-        int i; assert(P); i = parameterIndex - 1; if (P->paramCount <= 0 || \
-        i < 0 || i >= P->paramCount) THROW(SQLException, "Parameter index is out of range");
-
 extern const struct Rop_T postgresqlrops;
 
 
@@ -133,7 +131,8 @@ void PostgresqlPreparedStatement_free(T *P) {
 
 
 void PostgresqlPreparedStatement_setString(T P, int parameterIndex, const char *x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
         P->paramValues[i] = (char *)x;
         P->paramLengths[i] = 0;
         P->paramFormats[i] = 0;
@@ -141,7 +140,8 @@ void PostgresqlPreparedStatement_setString(T P, int parameterIndex, const char *
 
 
 void PostgresqlPreparedStatement_setInt(T P, int parameterIndex, int x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
         snprintf(P->params[i].s, 64, "%d", x);
         P->paramValues[i] =  P->params[i].s;
         P->paramLengths[i] = 0;
@@ -149,8 +149,9 @@ void PostgresqlPreparedStatement_setInt(T P, int parameterIndex, int x) {
 }
 
 
-void PostgresqlPreparedStatement_setLLong(T P, int parameterIndex, long long int x) {
-        TEST_INDEX
+void PostgresqlPreparedStatement_setLLong(T P, int parameterIndex, long long x) {
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
         snprintf(P->params[i].s, 64, "%lld", x);
         P->paramValues[i] =  P->params[i].s;
         P->paramLengths[i] = 0; 
@@ -159,7 +160,8 @@ void PostgresqlPreparedStatement_setLLong(T P, int parameterIndex, long long int
 
 
 void PostgresqlPreparedStatement_setDouble(T P, int parameterIndex, double x) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
         snprintf(P->params[i].s, 64, "%lf", x);
         P->paramValues[i] =  P->params[i].s;
         P->paramLengths[i] = 0;
@@ -167,8 +169,18 @@ void PostgresqlPreparedStatement_setDouble(T P, int parameterIndex, double x) {
 }
 
 
+void PostgresqlPreparedStatement_setTimestamp(T P, int parameterIndex, time_t x) {
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
+        P->paramValues[i] = Time_toString(x, P->params[i].s);
+        P->paramLengths[i] = 0;
+        P->paramFormats[i] = 0;
+}
+
+
 void PostgresqlPreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size) {
-        TEST_INDEX
+        assert(P);
+        int i = checkAndSetParameterIndex(parameterIndex, P->paramCount);
         P->paramValues[i] = (char *)x;
         P->paramLengths[i] = (x) ? size : 0;
         P->paramFormats[i] = 1;
@@ -179,7 +191,7 @@ void PostgresqlPreparedStatement_execute(T P) {
         assert(P);
         PQclear(P->res);
         P->res = PQexecPrepared(P->db, P->stmt, P->paramCount, (const char **)P->paramValues, P->paramLengths, P->paramFormats, 0);
-        P->lastError = PQresultStatus(P->res);
+        P->lastError = P->res ? PQresultStatus(P->res) : PGRES_FATAL_ERROR;
         if (P->lastError != PGRES_COMMAND_OK)
                 THROW(SQLException, "%s", PQresultErrorMessage(P->res));
 }
@@ -189,7 +201,7 @@ ResultSet_T PostgresqlPreparedStatement_executeQuery(T P) {
         assert(P);
         PQclear(P->res);
         P->res = PQexecPrepared(P->db, P->stmt, P->paramCount, (const char **)P->paramValues, P->paramLengths, P->paramFormats, 0);
-        P->lastError = PQresultStatus(P->res);
+        P->lastError = P->res ? PQresultStatus(P->res) : PGRES_FATAL_ERROR;
         if (P->lastError == PGRES_TUPLES_OK)
                 return ResultSet_new(PostgresqlResultSet_new(P->res, P->maxRows), (Rop_T)&postgresqlrops);
         THROW(SQLException, "%s", PQresultErrorMessage(P->res));
@@ -197,7 +209,7 @@ ResultSet_T PostgresqlPreparedStatement_executeQuery(T P) {
 }
 
 
-long long int PostgresqlPreparedStatement_rowsChanged(T P) {
+long long PostgresqlPreparedStatement_rowsChanged(T P) {
         assert(P);
         char *changes = PQcmdTuples(P->res);
         return changes ? Str_parseLLong(changes) : 0;
