@@ -51,10 +51,13 @@ const struct Rop_T postgresqlrops = {
         PostgresqlResultSet_free,
         PostgresqlResultSet_getColumnCount,
         PostgresqlResultSet_getColumnName,
-        PostgresqlResultSet_next,
         PostgresqlResultSet_getColumnSize,
+        PostgresqlResultSet_next,
+        PostgresqlResultSet_isnull,
         PostgresqlResultSet_getString,
         PostgresqlResultSet_getBlob,
+        NULL, // getTimestamp is handled in ResultSet
+        NULL  // getDateTime is handled in ResultSet
 };
 
 #define T ResultSetDelegate_T
@@ -66,10 +69,6 @@ struct T {
         PGresult *res;
 };
 
-#define TEST_INDEX \
-        int i; assert(R); i = columnIndex - 1; if (R->columnCount <= 0 || \
-        i < 0 || i >= R->columnCount) { THROW(SQLException, "Column index is out of range");}
-
 #define ISFIRSTOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '3')
 #define ISOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '7')
 #define OCTVAL(CH) ((CH) - '0')
@@ -166,14 +165,20 @@ int PostgresqlResultSet_getColumnCount(T R) {
 }
 
 
-const char *PostgresqlResultSet_getColumnName(T R, int column) {
+const char *PostgresqlResultSet_getColumnName(T R, int columnIndex) {
         assert(R);
-        column--;
-        if (R->columnCount <= 0 ||
-            column < 0          ||
-            column > R->columnCount)
+        columnIndex--;
+        if (R->columnCount <= 0 || columnIndex < 0 || columnIndex > R->columnCount)
                 return NULL;
-        return PQfname(R->res, column);
+        return PQfname(R->res, columnIndex);
+}
+
+
+long PostgresqlResultSet_getColumnSize(T R, int columnIndex) {
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        if (PQgetisnull(R->res, R->currentRow, i))
+                return 0;
+        return PQgetlength(R->res, R->currentRow, i);
 }
 
 
@@ -183,17 +188,17 @@ int PostgresqlResultSet_next(T R) {
 }
 
 
-long PostgresqlResultSet_getColumnSize(T R, int columnIndex) {
-        TEST_INDEX
-        if (PQgetisnull(R->res, R->currentRow, i)) 
-                return 0; 
-        return PQgetlength(R->res, R->currentRow, i);
+int PostgresqlResultSet_isnull(T R, int columnIndex) {
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        return PQgetisnull(R->res, R->currentRow, i);
 }
 
 
 const char *PostgresqlResultSet_getString(T R, int columnIndex) {
-        TEST_INDEX
-        if (PQgetisnull(R->res, R->currentRow, i)) 
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        if (PQgetisnull(R->res, R->currentRow, i))
                 return NULL; 
         return PQgetvalue(R->res, R->currentRow, i);
 }
@@ -206,8 +211,9 @@ const char *PostgresqlResultSet_getString(T R, int columnIndex) {
  * the buffer pointer. See also unescape_bytea() above.
  */
 const void *PostgresqlResultSet_getBlob(T R, int columnIndex, int *size) {
-        TEST_INDEX
-        if (PQgetisnull(R->res, R->currentRow, i)) 
+        assert(R);
+        int i = checkAndSetColumnIndex(columnIndex, R->columnCount);
+        if (PQgetisnull(R->res, R->currentRow, i))
                 return NULL; 
         return unescape_bytea((uchar_t*)PQgetvalue(R->res, R->currentRow, i), PQgetlength(R->res, R->currentRow, i), size);
 }
