@@ -26,6 +26,7 @@
 #include "Config.h"
 
 #include <stdio.h>
+#include <time.h>
 #include <sqlite3.h>
 
 #include "system/Time.h"
@@ -37,6 +38,8 @@
 
 /**
  * Implementation of the PreparedStatement/Delegate interface for SQLite 
+ * SQLite starts parameter index at 1, so checkAndSetParameterIndex is not 
+ * needed
  *
  * @file
  */
@@ -52,6 +55,7 @@ const struct Pop_T sqlite3pops = {
         SQLitePreparedStatement_setInt,
         SQLitePreparedStatement_setLLong,
         SQLitePreparedStatement_setDouble,
+        SQLitePreparedStatement_setTimestamp,
         SQLitePreparedStatement_setBlob,
         SQLitePreparedStatement_execute,
         SQLitePreparedStatement_executeQuery,
@@ -114,7 +118,7 @@ void SQLitePreparedStatement_setInt(T P, int parameterIndex, int x) {
 }
 
 
-void SQLitePreparedStatement_setLLong(T P, int parameterIndex, long long int x) {
+void SQLitePreparedStatement_setLLong(T P, int parameterIndex, long long x) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_int64(P->stmt, parameterIndex, x);
@@ -132,6 +136,12 @@ void SQLitePreparedStatement_setDouble(T P, int parameterIndex, double x) {
 }
 
 
+void SQLitePreparedStatement_setTimestamp(T P, int parameterIndex, time_t x) {
+        assert(P);
+        P->lastError = sqlite3_bind_int64(P->stmt, parameterIndex, x);
+}
+
+
 void SQLitePreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size) {
         assert(P);
         sqlite3_reset(P->stmt);
@@ -174,9 +184,9 @@ ResultSet_T SQLitePreparedStatement_executeQuery(T P) {
 }
 
 
-long long int SQLitePreparedStatement_rowsChanged(T P) {
+long long SQLitePreparedStatement_rowsChanged(T P) {
         assert(P);
-        return (long long int)sqlite3_changes(P->db);
+        return (long long)sqlite3_changes(P->db);
 }
 
 #ifdef PACKAGE_PROTECTED
