@@ -94,10 +94,9 @@ char *Str_copy(char *dest, const char *src, int n) {
 char *Str_dup(const char *s) { 
         char *t = NULL;
         if (s) {
-                size_t n = strlen(s); 
-                t = ALLOC(n + 1);
+                size_t n = strlen(s) + 1;
+                t = ALLOC(n);
                 memcpy(t, s, n);
-                t[n] = 0;
         }
         return t;
 }
@@ -162,15 +161,15 @@ int Str_parseInt(const char *s) {
 }
 
 
-long long int Str_parseLLong(const char *s) {
+long long Str_parseLLong(const char *s) {
 	if (STR_UNDEF(s))
 		THROW(SQLException, "NumberFormatException: For input string null");
         errno = 0;
         char *e;
-	long long l = strtoll(s, &e, 10);
+	long long ll = strtoll(s, &e, 10);
 	if (errno || (e == s))
 		THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
-	return l;
+	return ll;
 }
 
 
