@@ -27,6 +27,12 @@
 #define SCHEMA_SQLITE     "CREATE TABLE zild_t(id INTEGER PRIMARY KEY, name VARCHAR(255), percent REAL, image BLOB);"
 #define SCHEMA_ORACLE     "CREATE TABLE zild_t(id NUMBER , name VARCHAR(255), percent REAL, image CLOB);"
 
+#if HAVE_STRUCT_TM_TM_GMTOFF
+#define TM_GMTOFF tm_gmtoff
+#else
+#define TM_GMTOFF tm_wday
+#endif
+
 
 static void TabortHandler(const char *error) {
         fprintf(stdout, "Error: %s\n", error);
@@ -55,6 +61,7 @@ static void testPool(const char *testURL) {
                 exit(1);
         }
 
+        
         printf("=> Test1: create/destroy\n");
         {
                 pool = ConnectionPool_new(URL_new(testURL));
@@ -70,8 +77,17 @@ static void testPool(const char *testURL) {
         {
                 url = URL_new(NULL);
                 assert(! url);
-                pool = ConnectionPool_new(url);
-                assert(! pool);
+                TRY
+                {
+                        pool = ConnectionPool_new(url);
+                        printf("\tResult: Test failed -- exception not thrown\n");
+                        exit(1);
+                }
+                CATCH(AssertException)
+                {
+                        // OK
+                }
+                END_TRY;
         }
         printf("=> Test2: OK\n\n");
         
@@ -159,6 +175,7 @@ static void testPool(const char *testURL) {
                 // 2. Prepared statement, update the table proper with "images". 
                 PreparedStatement_T pre = Connection_prepareStatement(con, "update zild_t set image=? where id=?;");
                 assert(pre);
+                assert(PreparedStatement_getParameterCount(pre) == 2);
                 for (i = 0; images[i]; i++) {
                         PreparedStatement_setBlob(pre, 1, images[i], (int)strlen(images[i])+1);
                         PreparedStatement_setInt(pre, 2, i + 1);
@@ -166,11 +183,11 @@ static void testPool(const char *testURL) {
                 }
                 // The last execute changed one row only
                 assert(PreparedStatement_rowsChanged(pre) == 1);
-                /* Add a database null blob value */
+                /* Add a database null blob value for id = 5 */
                 PreparedStatement_setBlob(pre, 1, NULL, 0);
                 PreparedStatement_setInt(pre, 2, 5);
                 PreparedStatement_execute(pre);
-                /* Add a database null string value */
+                /* Add a database null string value for id = 1 */
                 PreparedStatement_setString(pre, 1, NULL);
                 PreparedStatement_setInt(pre, 2, 1);
                 PreparedStatement_execute(pre);
@@ -216,9 +233,11 @@ static void testPool(const char *testURL) {
                         const char *blob = (char*)ResultSet_getBlob(rset, 4, &imagesize);
                         printf("\t%-5d%-16s%-10.2f%-16.38s\n", id, name ? name : "null", percent, imagesize ? blob : "");
                 }
+                // Column count
                 rset = Connection_executeQuery(con, "select image from zild_t where id=12;");
-                assert(1==ResultSet_getColumnCount(rset));
-                // Assert that types are interchangeable (to some degree) and that all data is returned
+                assert(1 == ResultSet_getColumnCount(rset));
+                
+                // Assert that types are interchangeable and that all data is returned
                 while (ResultSet_next(rset)) {
                         const char *image = ResultSet_getStringByName(rset, "image");
                         const void *blob = ResultSet_getBlobByName(rset, "image", &imagesize);
@@ -226,6 +245,18 @@ static void testPool(const char *testURL) {
                         assert(strlen(image) + 1 == 8192);
                         assert(imagesize == 8192);
                 }
+                
+                printf("\tResult: check isnull..");
+                rset = Connection_executeQuery(con, "select id, image from zild_t where id in(1,5,2);");
+                while (ResultSet_next(rset)) {
+                        int id = ResultSet_getIntByName(rset, "id");
+                        if (id == 1 || id == 5)
+                                assert(ResultSet_isnull(rset, 2) == true);
+                        else
+                                assert(ResultSet_isnull(rset, 2) == false);
+                }
+                printf("success\n");
+
                 printf("\tResult: check max rows..");
                 Connection_setMaxRows(con, 3);
                 rset = Connection_executeQuery(con, "select id from zild_t;");
@@ -234,6 +265,7 @@ static void testPool(const char *testURL) {
                 while (ResultSet_next(rset)) i++;
                 assert((i)==3);
                 printf("success\n");
+                
                 printf("\tResult: check prepared statement resultset..");
                 Connection_setMaxRows(con, 0);
                 pre = Connection_prepareStatement(con, "select name from zild_t where id=?");
@@ -244,6 +276,7 @@ static void testPool(const char *testURL) {
                 assert(ResultSet_next(names));
                 assert(Str_isEqual("Leela", ResultSet_getString(names, 1)));
                 printf("success\n");
+                
                 printf("\tResult: check prepared statement re-execute..");
                 PreparedStatement_setInt(pre, 1, 1);
                 names = PreparedStatement_executeQuery(pre);
@@ -251,6 +284,7 @@ static void testPool(const char *testURL) {
                 assert(ResultSet_next(names));
                 assert(Str_isEqual("Fry", ResultSet_getString(names, 1)));
                 printf("success\n");
+                
                 printf("\tResult: check prepared statement without in-params..");
                 pre = Connection_prepareStatement(con, "select name from zild_t;");
                 assert(pre);
@@ -259,6 +293,7 @@ static void testPool(const char *testURL) {
                 for (i = 0; ResultSet_next(names); i++);
                 assert(i==12);
                 printf("success\n");
+                
                 /* Need to close and release statements before
                    we can drop the table, sqlite need this */
                 Connection_clear(con);
@@ -568,6 +603,80 @@ static void testPool(const char *testURL) {
                 }
         }
         printf("=> Test9: OK\n\n");
+        
+        printf("=> Test10: Date, Time, DateTime and Timestamp\n");
+        {
+                url = URL_new(testURL);
+                pool = ConnectionPool_new(url);
+                assert(pool);
+                setenv("TZ", "Europe/Oslo" , 1);
+                ConnectionPool_start(pool);
+                Connection_T con = ConnectionPool_getConnection(pool);
+                if (Str_startsWith(testURL, "postgres"))
+                        Connection_execute(con, "create table zild_t(d date, t time, dt timestamp, ts timestamp)");
+                else if (Str_startsWith(testURL, "oracle"))
+                        Connection_execute(con, "create table zild_t(d date, t time, dt date, ts timestamp)");
+                else
+                        Connection_execute(con, "create table zild_t(d date, t time, dt datetime, ts timestamp)");
+                PreparedStatement_T p = Connection_prepareStatement(con, "insert into zild_t values (?, ?, ?, ?)");
+                PreparedStatement_setString(p, 1, "2013-12-28");
+                PreparedStatement_setString(p, 2, "10:12:42");
+                PreparedStatement_setString(p, 3, "2013-12-28 10:12:42");
+                PreparedStatement_setTimestamp(p, 4, 1387066378);
+                PreparedStatement_execute(p);
+                ResultSet_T r = Connection_executeQuery(con, "select * from zild_t");
+                if (ResultSet_next(r)) {
+                        struct tm date = ResultSet_getDateTime(r, 1);
+                        struct tm time = ResultSet_getDateTime(r, 2);
+                        struct tm datetime = ResultSet_getDateTime(r, 3);
+                        time_t timestamp = ResultSet_getTimestamp(r, 4);
+                        struct tm timestampAsTm = ResultSet_getDateTime(r, 4);
+                        // Check Date
+                        assert(date.tm_hour == 0);
+                        assert(date.tm_year == 2013);
+                        assert(date.tm_mon == 11); // Remember month - 1
+                        assert(date.tm_mday == 28);
+                        assert(date.TM_GMTOFF == 0);
+                        // Check Time
+                        assert(time.tm_year == 0);
+                        assert(time.tm_hour == 10);
+                        assert(time.tm_min == 12);
+                        assert(time.tm_sec == 42);
+                        assert(time.TM_GMTOFF == 0);
+                        // Check datetime
+                        assert(datetime.tm_year == 2013);
+                        assert(datetime.tm_mon == 11); // Remember month - 1
+                        assert(datetime.tm_mday == 28);
+                        assert(datetime.tm_hour == 10);
+                        assert(datetime.tm_min == 12);
+                        assert(datetime.tm_sec == 42);
+                        assert(datetime.TM_GMTOFF == 0);
+                        // Check timestamp
+                        assert(timestamp == 1387066378);
+                        // Check timestamp as datetime
+                        assert(timestampAsTm.tm_year == 2013);
+                        assert(timestampAsTm.tm_mon == 11); // Remember month - 1
+                        assert(timestampAsTm.tm_mday == 15);
+                        assert(timestampAsTm.tm_hour == 0);
+                        assert(timestampAsTm.tm_min == 12);
+                        assert(timestampAsTm.tm_sec == 58);
+                        assert(timestampAsTm.TM_GMTOFF == 0);
+                        // Result
+                        printf("\tResult: Date: %s, Time: %s, DateTime: %s, Timestamp: %s\n",
+                               ResultSet_getString(r, 1),
+                               ResultSet_getString(r, 2),
+                               ResultSet_getString(r, 3),
+                               ResultSet_getString(r, 4)); // SQLite will show unix time, others will show a time string
+                }
+                Connection_execute(con, "drop table zild_t;");
+                Connection_close(con);
+                ConnectionPool_stop(pool);
+                ConnectionPool_free(&pool);
+                assert(pool==NULL);
+                URL_free(&url);
+        }
+        printf("=> Test10: OK\n\n");
+
 
         printf("============> Connection Pool Tests: OK\n\n");
 }
