@@ -2,6 +2,7 @@
 #include <assert.h>
 #include <string.h>
 #include <fcntl.h>
+#include <stdlib.h>
 
 #include "Config.h"
 #include "URL.h"
@@ -93,21 +94,21 @@ static void testStr() {
         }
         printf("=> Test5: OK\n\n");
         
-        printf("=> Test6: parseInt, parseLLong, parseDouble\n");
+        printf("=> Test6: parseInt, parseLong, parseLLong, parseFloat, parseDouble\n");
         {
                 char i[STRLEN] = "   -2812 bla";
                 char ll[STRLEN ] = "  2147483642 blabla";
-                char d[STRLEN] = "  2.718281828 this is e";
-                char de[STRLEN] = "1.495E+08 kilometer = An Astronomical Unit";
+                char d[STRLEN] = "  2.71828182845904523536028747135266249775724709369995 this is e";
+                char de[STRLEN] = "9.461E^99 nanometers";
                 char ie[STRLEN] = " 9999999999999999999999999999999999999";
                 printf("\tResult:\n");
-                printf("\tParsed int = %d\n", Str_parseInt(i));
-                printf("\tParsed long long = %lld\n", Str_parseLLong(ll));
-                printf("\tParsed double = %.9f\n", Str_parseDouble(d));
+                printf("\tParsed int32 = %d\n", Str_parseInt(i));
+                printf("\tParsed int64 = %lld\n", Str_parseLLong(ll));
+                printf("\tParsed double = %.52f\n", Str_parseDouble(d));
                 printf("\tParsed double exp = %.3e\n", Str_parseDouble(de));
                 TRY
                 {
-                        printf("\tParse truncated int = %d\n", Str_parseInt(ie));
+                        printf("\tParse overflow int = %d\n", Str_parseInt(ie));
                         assert(false); //Should not come here
                 }
                 CATCH(SQLException)
@@ -177,11 +178,147 @@ static void testTime() {
         }
         printf("=> Test2: OK\n\n");
         
-        printf("=> Test3: usleep\n");
+        printf("=> Test3: Time_toString\n");
+        {
+                char *t = Time_toString(1387010338, (char[20]){0});
+                assert(Str_isEqual(t, "2013-12-14 08:38:58"));
+                printf("\tResult: %s\n", t);
+                // Assert time is converted in UTC timezone
+                Time_toString(0, t);
+                assert(Str_isEqual(t, "1970-01-01 00:00:00"));
+                printf("\tResult: %s\n", t);
+        }
+        printf("=> Test3: OK\n\n");
+        
+        printf("=> Test4: Time_toDateTime\n");
+        {
+#if HAVE_STRUCT_TM_TM_GMTOFF
+#define TM_GMTOFF tm_gmtoff
+#else
+#define TM_GMTOFF tm_wday
+#endif
+                struct tm t;
+                // DateTime ISO-8601 format
+                assert(Time_toDateTime("2013-12-14T09:38:08Z", &t));
+                assert(t.tm_year == 2013);
+                assert(t.tm_mon  == 11);
+                assert(t.tm_mday == 14);
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                // Date
+                assert(Time_toDateTime("2013-12-14", &t));
+                assert(t.tm_year == 2013);
+                assert(t.tm_mon  == 11);
+                assert(t.tm_mday == 14);
+                // Time
+                assert(Time_toDateTime("09:38:08", &t));
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                // Compressed DateTime
+                assert(Time_toDateTime(" 20131214093808", &t));
+                assert(t.tm_year == 2013);
+                assert(t.tm_mon  == 11);
+                assert(t.tm_mday == 14);
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                // Compressed Date
+                assert(Time_toDateTime(" 20131214 ", &t));
+                assert(t.tm_year == 2013);
+                assert(t.tm_mon  == 11);
+                assert(t.tm_mday == 14);
+                // Compressed Time
+                assert(Time_toDateTime("093808", &t));
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                // Reverse DateTime
+                assert(Time_toDateTime(" 09:38:08 2013-12-14", &t));
+                assert(t.tm_year == 2013);
+                assert(t.tm_mon  == 11);
+                assert(t.tm_mday == 14);
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                // DateTime with timezone Zulu (UTC)
+                assert(Time_toDateTime("The Battle of Stamford Bridge 1066-09-25 12:15:33+00:00", &t));
+                assert(t.tm_year == 1066);
+                assert(t.tm_mon  == 8);
+                assert(t.tm_mday == 25);
+                assert(t.tm_hour == 12);
+                assert(t.tm_min  == 15);
+                assert(t.tm_sec  == 33);
+                assert(t.TM_GMTOFF == 0); // offset from UTC in seconds
+                // Time with timezone
+                assert(Time_toDateTime(" 09:38:08+01:45", &t));
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                assert(t.TM_GMTOFF == 6300);
+                // Time with timezone PST compressed
+                assert(Time_toDateTime("Pacific Time Zone 09:38:08 -0800 ", &t));
+                assert(t.tm_hour == 9);
+                assert(t.tm_min  == 38);
+                assert(t.tm_sec  == 8);
+                assert(t.TM_GMTOFF == -28800);
+                // Date without time, tz should not be set
+                assert(Time_toDateTime("2013-12-15-0800 ", &t));
+                assert(t.TM_GMTOFF == 0);
+                // Invalid date
+                TRY {
+                        Time_toDateTime("1901-123-45", &t);
+                        printf("\t Test Failed\n");
+                        exit(1);
+                } CATCH (SQLException) {
+                        // OK
+                } ELSE {
+                        printf("\t Test Failed with wrong exception\n");
+                        exit(1);
+                }
+                END_TRY;
+        }
+        printf("=> Test4: OK\n\n");
+        
+        printf("=> Test5: Time_toTimestamp\n");
+        {
+                // Time, fraction of second is ignored
+                time_t t = Time_toTimestamp("2013-12-15 00:12:58.123456");
+                assert(t == 1387066378);
+                // TimeZone east, ignored. String is parsed as UTC
+                t = Time_toTimestamp("Tokyo timezone: 2013-12-15 00:12:58+09:00");
+                assert(t == 1387066378);
+                // TimeZone west, ignored. String is parsed as UTC
+                t = Time_toTimestamp("New York timezone: 2013-12-15 00:12:58-05:00");
+                assert(t == 1387066378);
+                // TimeZone Zulu
+                t = Time_toTimestamp("Grenwich timezone: 2013-12-15 00:12:58Z");
+                assert(t == 1387066378);
+                // Compressed
+                t = Time_toTimestamp("20131215001258-0500");
+                assert(t == 1387066378);
+                // Invalid timestamp string
+                TRY {
+                        Time_toTimestamp("1901-123-45 10:12:14");
+                        // Should not come here
+                        printf("\t Test Failed\n");
+                        exit(1);
+                } CATCH (SQLException) {
+                        // OK
+                } ELSE {
+                        printf("\t Test Failed with wrong exception\n");
+                        exit(1);
+                }
+                END_TRY;
+        }
+        printf("=> Test5: OK\n\n");
+        
+        printf("=> Test6: usleep\n");
         {
                 Time_usleep(1);
         }
-        printf("=> Test3: OK\n\n");
+        printf("=> Test6: OK\n\n");
         
         printf("============> Time Tests: OK\n\n");
 }
@@ -601,7 +738,7 @@ static void testStringBuffer() {
         }
         printf("=> Test1: OK\n\n");
         
-        printf("=> Test2: Append NULL value\n");
+        printf("=> Test2: append NULL value\n");
         {
                 sb = StringBuffer_new("");
                 assert(sb);
@@ -612,7 +749,7 @@ static void testStringBuffer() {
         }
         printf("=> Test2: OK\n\n");
         
-        printf("=> Test3: Create with string\n");
+        printf("=> Test3: create with string\n");
         {
                 sb = StringBuffer_new("abc");
                 assert(sb);
@@ -622,7 +759,7 @@ static void testStringBuffer() {
         }
         printf("=> Test3: OK\n\n");
         
-        printf("=> Test4: Append string value\n");
+        printf("=> Test4: append string value\n");
         {
                 sb = StringBuffer_new("abc");
                 assert(sb);
@@ -752,6 +889,27 @@ static void testStringBuffer() {
         }
         printf("=> Test7: OK\n\n");
         
+        printf("=> Test8: set value\n");
+        {
+                sb= StringBuffer_new("abc");
+                assert(sb);
+                StringBuffer_set(sb, "def");
+                assert(IS(StringBuffer_toString(sb), "def"));
+                StringBuffer_free(&sb);
+                assert(sb==NULL);
+                printf("\tTesting set and internal resize:...");
+                sb= StringBuffer_create(4);
+                assert(sb);
+                StringBuffer_set(sb, "abc");
+                StringBuffer_set(sb, "abcdef");
+                assert(IS(StringBuffer_toString(sb), "abcdef"));
+                StringBuffer_set(sb, " ");
+                assert(IS(StringBuffer_toString(sb), " "));
+                printf("ok\n");
+                StringBuffer_free(&sb);
+                assert(sb==NULL);
+        }
+        printf("=> Test8: OK\n\n");
 
         printf("============> StringBuffer Tests: OK\n\n");
 }
@@ -759,6 +917,7 @@ static void testStringBuffer() {
 
 int main(void) {
         Exception_init();
+        setenv("TZ", "Europe/Oslo" , 1);
 	testStr();
 	testMem();
 	testTime();
